package bubble.model.device;

import bubble.model.account.Account;
import bubble.model.account.HasAccount;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import lombok.ToString;
import lombok.experimental.Accessors;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.map.SingletonMap;
import org.cobbzilla.util.collection.ArrayUtil;
import org.cobbzilla.wizard.model.Identifiable;
import org.cobbzilla.wizard.model.IdentifiableBase;
import org.cobbzilla.wizard.model.entityconfig.EntityFieldMode;
import org.cobbzilla.wizard.model.entityconfig.EntityFieldType;
import org.cobbzilla.wizard.model.entityconfig.annotations.*;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Transient;

import static bubble.ApiConstants.EP_FLEX_ROUTERS;
import static org.cobbzilla.util.daemon.ZillaRuntime.*;
import static org.cobbzilla.util.json.JsonUtil.json;
import static org.cobbzilla.util.reflect.ReflectionUtil.copy;

@Entity
@ECType(root=true) @ToString(of={"ip", "port"})
@ECTypeURIs(baseURI=EP_FLEX_ROUTERS, listFields={"name", "enabled"})
@NoArgsConstructor @Accessors(chain=true) @Slf4j
@ECIndexes({ @ECIndex(unique=true, of={"account", "ip"}) })
public class FlexRouter extends IdentifiableBase implements HasAccount {

    public static final String[] UPDATE_FIELDS = { "enabled", "active", "proxy_port", "auth_token" };
    public static final String[] CREATE_FIELDS = ArrayUtil.append(UPDATE_FIELDS, "ip");

    public FlexRouter (FlexRouter other) { copy(this, other, CREATE_FIELDS); }

    @Override public Identifiable update(Identifiable other) { copy(this, other, UPDATE_FIELDS); return this; }

    @ECSearchable(filter=true) @ECField(index=10)
    @ECIndex @Column(nullable=false, length=500)
    @Getter @Setter private String ip;

    @JsonIgnore @Transient public String getName () { return getIp(); }

    @ECSearchable(filter=true) @ECField(index=20)
    @ECIndex @Column(nullable=false)
    @JsonIgnore @Getter @Setter private Integer port = 0;
    public boolean hasPort () { return port != null && port > 1024; }

    // used for sending the port, we never send it back
    @Transient @Getter @Setter private Integer proxy_port;
    public boolean hasProxyPort () { return proxy_port != null && proxy_port > 1024; }

    public String id () { return getIp() + "/" + getUuid(); }

    @ECSearchable @ECField(index=30)
    @ECForeignKey(entity=Account.class)
    @Column(nullable=false, updatable=false, length=UUID_MAXLEN)
    @Getter @Setter private String account;

    @ECSearchable @ECField(index=40)
    @ECIndex @Column(nullable=false)
    @Getter @Setter private Boolean enabled = true;
    public boolean enabled () { return bool(enabled); }

    @ECSearchable @ECField(index=50)
    @ECIndex @Column(nullable=false)
    @Getter @Setter private Boolean active = true;
    public boolean active() { return bool(active); }
    public boolean inactive() { return !active(); }

    @ECSearchable @ECField(index=60, type=EntityFieldType.epoch_time, mode=EntityFieldMode.readOnly)
    @Getter @Setter private Long lastSeen;
    public FlexRouter setLastSeen () { return setLastSeen(now()); }

    @JsonIgnore @Transient public long getAge () { return lastSeen == null ? Long.MAX_VALUE : now() - lastSeen; }

    @ECSearchable(filter=true) @ECField(index=70)
    @Column(length=100)
    @JsonIgnore @Getter @Setter private String token;
    public boolean hasToken () { return !empty(token); }

    // used for sending the token, we never send it back
    @Transient @Getter @Setter private String auth_token;
    public boolean hasAuthToken () { return !empty(auth_token); }

    public String pingUrl() { return "http://" + getIp() + ":" + getPort() + "/ping"; }
    public String pingObject() { return json(new SingletonMap("token", getToken())); }

}
